package cn.rankin.cmsweb.configuration;

import cn.rankin.cmsweb.code.CmsWebAPICode;
import cn.rankin.cmsweb.entity.UserDetails;
import cn.rankin.cmsweb.security.CmsAuthenticationProvider;
import cn.rankin.cmsweb.security.CmsUserDetailsService;
import cn.rankin.cmsweb.security.JwtAuthenticationTokenFilter;
import cn.rankin.cmsweb.security.JwtTokenService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.util.HttpUtil;
import com.google.common.collect.ImmutableList;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.security.SecurityProperties;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.RedisOperations;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.builders.WebSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationFailureHandler;
import org.springframework.security.web.authentication.SimpleUrlAuthenticationSuccessHandler;
import org.springframework.security.web.authentication.logout.SimpleUrlLogoutSuccessHandler;
import org.springframework.security.web.authentication.session.SessionAuthenticationException;
import org.springframework.security.web.header.Header;
import org.springframework.security.web.header.HeaderWriter;
import org.springframework.security.web.session.SessionManagementFilter;
import org.springframework.session.data.redis.RedisOperationsSessionRepository;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 安全配置中心
 *
 * @author by tomas
 * @date 2017-06-06 15:30:00
 */
@Configuration
@EnableWebSecurity
@EnableGlobalMethodSecurity(prePostEnabled = true)//允许进入页面方法前检验
@Order(SecurityProperties.ACCESS_OVERRIDE_ORDER)
public class SecurityConfig extends WebSecurityConfigurerAdapter {

    private static final Logger logger = LoggerFactory.getLogger(SecurityConfig.class);

    @Value("${jwt.header:X-Token}")
    private String tokenHeader;

    @Value("${jwt.cookie:X-Cookie}")
    private String cookieName;

    @Value("${jwt.expiration:604800}")
    private Integer expiration;

    @Autowired
    private JwtTokenService jwtTokenService;

    @Autowired
    private JwtAuthenticationTokenFilter jwtAuthenticationTokenFilter;

    public static class StaticHeadersWriter implements HeaderWriter {

        private final static List<Header> headers = new ArrayList<>();

        static {
            headers.add(new Header("Access-Control-Allow-Origin", "*"));
            headers.add(new Header("Access-Control-Allow-Methods", "GET, POST, HEAD, OPTIONS, PUT, DELETE, TRACE, PATCH"));
            headers.add(new Header("Access-Control-Allow-Headers", "Origin, X-Token, Authentication, Accept, X-Requested-With, Content-Type, Access-Control-Request-Method, Access-Control-Request-Headers, Authorization, Cache-control "));
            headers.add(new Header("Access-Control-Allow-Credentials", "true"));
        }

        @Override
        public void writeHeaders(HttpServletRequest request, HttpServletResponse response) {
            for (Header header : headers) {
                for (String value : header.getValues()) {
                    response.addHeader(header.getName(), value);
                }
            }

            String origin = request.getHeader("Origin");
            if (!StringUtils.isEmpty(origin)) {
                response.setHeader("Access-Control-Allow-Origin", origin);
            }
        }
    }


    /**
     * 用户查询服务
     *
     * @return
     */
    @Bean
    public UserDetailsService userDetailsService() {
        return new CmsUserDetailsService();
    }

    /**
     * 用户认证 provider
     *
     * @return
     */
    @Bean
    public CmsAuthenticationProvider cmsAuthenticationProvider() {
        return new CmsAuthenticationProvider();
    }

    /**
     * 设置那些 URL 忽略权限
     *
     * @param web
     * @throws Exception
     */
    @Override
    public void configure(WebSecurity web) throws Exception {
        HttpUtil.antMatchers(HttpMethod.OPTIONS);
        HttpUtil.antMatchers(HttpMethod.GET, "/error", "/login/**", "/logout", "/favicon.ico");
        web.ignoring().requestMatchers(HttpUtil.getMatchersArray());
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {

        // 禁用缓存
        http.
                headers().cacheControl();
//                .and().addHeaderWriter(new StaticHeadersWriter());

        http
                // 分权限验证
                .authorizeRequests()
                .anyRequest().authenticated()
                .and().csrf().disable()
                .authorizeRequests().antMatchers("/login", "/logout", "/favicon.ico", "/error", "/file/**").permitAll()
                .and().addFilterBefore(jwtAuthenticationTokenFilter, SessionManagementFilter.class);

        http
                .formLogin().successHandler(new RestAuthenticationSuccessHandler())
                .failureHandler(new RestAuthenticationFailureHandler())
                .permitAll()
                .and()
                .logout().logoutSuccessHandler(new RestLogoutSuccessHandler())
                .deleteCookies("JSESSIONID", "JWT_TOKEN")
                .permitAll();
        //session 管理
        http.sessionManagement().maximumSessions(5);

        http.exceptionHandling().authenticationEntryPoint(new RestAuthenticationEntryPoint());

    }

    /**
     * 自定义 认证密码 和 获取用户 服务
     *
     * @param auth
     * @throws Exception
     */
    @Autowired
    public void configureGlobal(AuthenticationManagerBuilder auth) throws Exception {
        auth
                .authenticationProvider(cmsAuthenticationProvider())
                .userDetailsService(userDetailsService());
    }

    @Bean
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

    /**
     * Rest 登陆成功后的处理
     */
    public class RestAuthenticationSuccessHandler extends SimpleUrlAuthenticationSuccessHandler {
        @Override
        public void onAuthenticationSuccess(HttpServletRequest request,
                                            HttpServletResponse response, Authentication authentication)
                throws ServletException, IOException {

            if (null != authentication.getPrincipal()) {
                UserDetails userDetails = (UserDetails) authentication.getPrincipal();
                //兼容 cookie 和 header x-token 验证两种方式
                Cookie cookie = new Cookie(cookieName, jwtTokenService.generateToken(userDetails.getUsername()));
                cookie.setPath("/");
                cookie.setMaxAge(expiration);
                cookie.isHttpOnly();
                response.addCookie(cookie);
                response.addHeader(tokenHeader, jwtTokenService.generateToken(userDetails.getUsername()));
                //redisService.set(String.format(LOGIN_TOKEN_FORMAT_KEY,userDetails.getUsername()),userDetails,expiration, TimeUnit.SECONDS);
                HttpUtil.responseOutWithJson(request, response, APIResult.ok(userDetails));
                clearAuthenticationAttributes(request);
            }

        }
    }

    /**
     * Rest 登录认证失败后的处理
     */

    public class RestAuthenticationFailureHandler extends SimpleUrlAuthenticationFailureHandler {
        @Override
        public void onAuthenticationFailure(HttpServletRequest request, HttpServletResponse response, AuthenticationException exception)
                throws ServletException, IOException {

            logger.error("登录失败 exception={}", exception);
            saveException(request, exception);

            //密码错误
            if (exception instanceof BadCredentialsException) {
                HttpUtil.error(request, response, CmsWebAPICode.PASSWORD_ERROR);
            }
            //查询用户出错
            else if (exception instanceof UsernameNotFoundException) {
                HttpUtil.error(request, response, CmsWebAPICode.USERNAME_PASSWORD_ERROR);
            }
            //查询用户出错
            else if (exception instanceof SessionAuthenticationException) {
                HttpUtil.error(request, response, CmsWebAPICode.MAX_COUNT_SESSION_ERROR);
            } else {
                HttpUtil.error(request, response, CmsWebAPICode.AUTHORIZED_FAILD);
            }
        }
    }

    /**
     * Rest 登出成功后的处理
     */

    public class RestLogoutSuccessHandler extends SimpleUrlLogoutSuccessHandler {
        @Override
        public void onLogoutSuccess(HttpServletRequest request,
                                    HttpServletResponse response, Authentication authentication)
                throws IOException, ServletException {

            logger.info("登出成功! ");
            HttpUtil.delCookies(request, response, "JSESSIONID", cookieName);
            HttpUtil.ok(request, response);

        }
    }

    /**
     * Rest 权限不通过的处理
     */
    public class RestAuthenticationEntryPoint implements AuthenticationEntryPoint {
        @Override
        public void commence(HttpServletRequest request,
                             HttpServletResponse response,
                             AuthenticationException authException) throws IOException {
            //原生返回 response.sendError(HttpServletResponse.SC_UNAUTHORIZED,"Authentication Failed: " + authException.getMessage());
            //cookie失效
            HttpUtil.error(request, response, CmsWebAPICode.AUTHORIZED_FAILD);
        }
    }

    @Bean
    public RedisOperationsSessionRepository sessionRepository(@Qualifier("sessionRedisTemplate") RedisOperations<Object, Object> sessionRedisTemplate,
                                                              ApplicationEventPublisher applicationEventPublisher) {
        RedisOperationsSessionRepository sessionRepository = new RedisOperationsSessionRepository(sessionRedisTemplate);
        sessionRepository.setApplicationEventPublisher(applicationEventPublisher);
        sessionRepository.setDefaultMaxInactiveInterval(expiration);
        return sessionRepository;
    }
}
