package cn.rankin.common.utils.service.impl;

import cn.rankin.common.utils.configuration.RedisConfiguration;
import cn.rankin.common.utils.service.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Import;
import org.springframework.data.redis.core.*;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.concurrent.TimeUnit;

@Slf4j
@Component
@Import(RedisConfiguration.class)
public class RedisServiceImpl<K,V> implements RedisService<K,V> {


	@Autowired
	@Qualifier(value = "redisTemplate")
	RedisTemplate<K,V> redisTemplate;

	public void setRedisTemplate(RedisTemplate redisTemplate) {
	    this.redisTemplate = redisTemplate;
    }

	/**
	 * 获取
	 * @param key
	 * @return Object
	 */
	@Override
	public V get(K key) {
		return redisTemplate.opsForValue().get(key);
	}

	/**
	 * 默认设置 30天有效 避免长期 暂用 redis
	 * @param key
	 * @param value
	 */
	@Override
	public void set(K key, V value) {
		redisTemplate.opsForValue().set(key, value,3600*24*30, TimeUnit.SECONDS);
	}

	/**
	 * 存值并且设置 redis key 过期时间
	 * @param key
	 * @param value
	 * @param timeout
	 */
	@Override
	public void set(K key, V value, long timeout) {
		redisTemplate.opsForValue().set(key, value, timeout, TimeUnit.SECONDS);
	}

	/**
	 * 存值并且设置 redis key 过期时间
	 * @param key
	 * @param value
	 * @param timeout
	 */
	@Override
	public void set(K key, V value, long timeout,TimeUnit unit) {
		redisTemplate.opsForValue().set(key, value, timeout, unit);
	}

	/**
	 * 删除 key
	 * @param key
	 * @return Object
	 */
	@Override
	public void delete(K key) {
		if (hasKey(key)) {
			redisTemplate.delete(key);
		}
	}

	/**
	 * 删除 key
	 *
	 * @param keys
	 * @return Object
	 */
	@Override
	public void delete(Collection<K> keys) {
		redisTemplate.delete(keys);
	}

	/**
	 * 剩余生存时间 (秒单位)
	 * @param key
	 * @return
	 */
	@Override
	public Long getExpire(K key) {
		if (!hasKey(key)) {
			return 0L;
		}
		return redisTemplate.getExpire(key);
	}

	/**
	 * 更新 key 的过期时间
	 * @param key
	 * @param timeout
	 * @param unit
	 * @return
	 */
	@Override
	public boolean setExpire(K key, long timeout, TimeUnit unit) {
		return redisTemplate.expire(key, timeout, unit);
	}

	/**
	 * 判断 key 是否存在
	 * @param key
	 * @return
	 */
	@Override
	public boolean hasKey(K key) {
		return redisTemplate.hasKey(key);
	}

	/**
	 * 返回 redisTemplate for Hash的原生 操作对象
	 * @return HashOperations
	 */
	@Override
	public HashOperations redisHash() {
		return redisTemplate.opsForHash();
	}

	/**
	 * 返回 redisTemplate for List的原生 操作对象
	 * @return ListOperations
	 */
	@Override
	public ListOperations redisList() {
		return redisTemplate.opsForList();
	}
	/**
	 * 返回 redisTemplate for Set的原生 操作对象
	 * @return SetOperations
	 */
	@Override
	public SetOperations redisSet()  {
		return redisTemplate.opsForSet();
	}

	/**
	 * 返回 redisTemplate for ZSet的原生 操作对象
	 * @return ZSetOperations
	 */
	@Override
	public ZSetOperations redisZSet()  {
		return redisTemplate.opsForZSet();
	}
	/**
	 * 返回 redisTemplate for Value的原生 操作对象
	 * @return ValueOperations
	 */

	@Override
	public ValueOperations redisValue()  {
		return redisTemplate.opsForValue();
	}

	/**
	 * /**
	 * 返回 redisTemplat 原生操作对象
	 * return redisTemplate
	 */
	@Override
	public RedisTemplate redisTemplate()  {
		return redisTemplate;
	}


}