package cn.rankin.productservice.controller;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.dto.PosterDTO;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.ProductRepository;
import cn.rankin.productservice.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.LinkedHashMap;
import java.util.List;

@Slf4j
@RestController
@RequestMapping(value = "poster")
public class PosterController {

    @Autowired
    private PosterService posterService;

    @Autowired
    private ProductRepository productRepository;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<List<Poster>> search(PosterDTO searchDTO) {
        Poster poster = new Poster();

        poster.setStatus(searchDTO.getStatus());
        poster.setMerchantId(searchDTO.getMerchantId());
        poster.setPid(searchDTO.getPid());
        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap() { {
            this.put("gmtModified", BaseOrderEnum.DESC);
        }};

        List<Poster> posters = posterService.search(poster , sort);
        posters.forEach(entity ->{
            setProductInfo(entity);
        } );
        return APIResult.ok(posters);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<Poster> getPoster(@PathVariable("id") String id) {
        if (StringUtils.isEmpty(id)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        Poster result = posterService.findById(id);
        if(null == result){
            return APIResult.error(APICode.NOT_EXISTS);
        }

        setProductInfo(result);
        return APIResult.ok(result);
    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<Poster> create(@Valid @RequestBody PosterDTO posterDTO) {
        if(isExist(posterDTO)){
            return APIResult.error(APICode.ALREADY_EXISTS);
        }
        // 创建海报
        Poster result = posterService.create(posterDTO);
        if (null == result) {
            return APIResult.error(APICode.OPERATE_ERROR);
        }
        setProductInfo(result);
        return APIResult.ok(result);
    }

    private boolean isExist(PosterDTO posterDTO) {
        return posterService.isExist(posterDTO);
    }

    @Transactional
    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<Poster> update(@Valid @RequestBody PosterDTO posterDTO) {
        String Id = posterDTO.getId();
        if (Id == null) {
            return APIResult.error(APICode.error("参数错误: id不能为空"));
        }
        if(isExist(posterDTO)){
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        Poster result = posterService.update(posterDTO);
        if (null == result) {
            return APIResult.error(APICode.OPERATE_ERROR);
        }

        setProductInfo(result);
        return APIResult.ok(result);

    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult delete(@PathVariable("id") String id) {
        Boolean flag = posterService.delete(id);
        if(flag){
            return APIResult.ok();
        }else{
            return APIResult.error(ProductServiceAPICode.error("删除失败"));
        }
    }

    @RequestMapping(value = "/updatePosters/{merchantId}", method = RequestMethod.PUT)
    public APIResult<List<Poster>> updatePosters(@RequestParam("merchantId") String merchantId,
                                          @RequestBody List<String> posterIdList){
        List<Poster> posters = posterService.updatePosters(merchantId, posterIdList);

        posters.forEach(poster ->{
            setProductInfo(poster);
        });
        return APIResult.ok(posters);
    }

    @RequestMapping(value = "/getPosters/{merchantId}", method = RequestMethod.GET)
    public APIResult<List<Poster>> getPosters(@RequestParam("merchantId") String merchantId){
        List<Poster> posters = posterService.getByMerchantId(merchantId);

        posters.forEach(poster ->{
            setProductInfo(poster);
        });
        return APIResult.ok(posters);
    }

    /**
     * 组装产品信息
     * @param poster
     */
    private void setProductInfo(Poster poster) {
        if(null == poster){
            return ;
        }
        Product product = productRepository.findByPid(poster.getPid());
        poster.setProduct(product);
    }


}
