package cn.rankin.productservice.controller.cms;

import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.dto.search.SearchDTO;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.LessonDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.productservice.service.*;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

@RestController
@RequestMapping(value = "lesson")
public class LessonController {

    @Autowired
    private CourseService courseService;

    @Autowired
    private CourseSubRelationService courseSubRelationService;

    @Autowired
    private LessonService lessonService;

    @Autowired
    private LessonWareRelationService lessonWareRelationService;

    @Autowired
    private WareService wareService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<Lesson>> search(SearchDTO searchDTO) {
        Lesson lesson = new Lesson();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            lesson.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            lesson.setTitle("%" + name + "%");
        }

        lesson.setStatus(searchDTO.getStatus());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap() { {
            this.put("gmtModified", BaseOrderEnum.DESC);
        }};

        return lessonService.search(lesson, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<Lesson> getLesson(@PathVariable("id") String id) {

        if (StringUtils.isEmpty(id)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        APIResult<Lesson> result = lessonService.findById(id);
        Lesson lesson = result.getData();
        if (!result.getSuccess() || lesson == null) {
            return result;
        }

        List<LessonWareRelation> relationList = lessonWareRelationService.findByLessonId(lesson.getId());
        if (relationList == null || relationList.size() == 0) {
            return result;
        }

        List<String> wareIdList = new ArrayList() {
            {
                relationList.forEach( e -> add(e.getWareId()));
            }
        };

        APIResult<List<CourseWare>> wareList = wareService.findByIds(wareIdList);
        lesson.setWareList(wareList.getData());

        return result;

    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<Lesson> create(@Valid @RequestBody LessonDTO lessonDTO) {
        String code = lessonDTO.getCode();
        if (lessonDTO.getId() != null || StringUtils.isEmpty(code)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        // 更新课
        APIResult<Lesson> result = lessonService.create(lessonDTO);
        if (!result.getSuccess()) {
            return result;
        }

        String Id = result.getData().getId();

        // 为空不更新
        List<String> wareIdList = lessonDTO.getWareList();
        if (wareIdList == null) {
            return result;
        }

        APIResult<List<CourseWare>> wareResult = updateRelation(Id, wareIdList);
        if (!wareResult.getSuccess()) {
            // 显示回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return APIResult.error(new BaseCode(wareResult.getCode(), wareResult.getMessage()));
        }

        List<CourseWare> courseWareList = wareResult.getData();
        if (courseWareList != null) {
            result.getData().setWareList(courseWareList);
        }

        return result;
    }

    @Transactional
    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<Lesson> update(@Valid @RequestBody LessonDTO lessonDTO) {
        String Id = lessonDTO.getId();
        if (Id == null) {
            return APIResult.error(APICode.error("参数错误: id不能为空"));
        }

        List<String> wareIdList = lessonDTO.getWareList();
        List<CourseWare> courseWareList = new ArrayList<>();
        if (wareIdList != null) {
            APIResult<List<CourseWare>> wareResult = updateRelation(Id, wareIdList);
            if (!wareResult.getSuccess()) {
                return APIResult.error(new BaseCode(wareResult.getCode(), wareResult.getMessage()));
            }else {
                courseWareList = wareResult.getData();
            }
        }

        APIResult<Lesson> result = lessonService.update(lessonDTO);
        if (result.getSuccess() && result.getData() != null) {
            result.getData().setWareList(courseWareList);
        }

        return result;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult delete(@PathVariable("id") String id) {
        if (StringUtils.isEmpty(id)) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        List<CourseSubRelation> courseSubRelationList = courseSubRelationService.findByLessonId(id);
        if (courseSubRelationList == null || courseSubRelationList.size() != 0) {
            return lessonService.delete(id);
        }

        List<String> courseIdList = new ArrayList<>();
        courseSubRelationList.forEach( relation -> {
            String courseId = relation.getCourseId();
            if (!courseIdList.contains(courseId)) {
                courseIdList.add(courseId);
            }
        });

        APIResult<List<Course>> courseResult = courseService.findByIds(courseIdList);
        APIResult<List<Course>> result = APIResult.error(APICode.CAN_NOT_DEL);
        result.setData(courseResult.getData());

        return result;

    }

    // 更新课与课件的关系
    @Transactional
    public APIResult<List<CourseWare>> updateRelation(String lessonId, List<String> wareIdList) {
        if (wareIdList == null) {
            return APIResult.ok();
        }

        List<CourseWare> courseWareList = new ArrayList<>();
        if (wareIdList.size() == 0) {
            lessonWareRelationService.deleteByLessonId(lessonId);
        }else {
            // 验证wareId的正确性
            APIResult<List<CourseWare>> wareResult = wareService.findByIds(wareIdList);
            courseWareList = wareResult.getData();
            if (courseWareList == null || courseWareList.size() != wareIdList.size()) {
                return APIResult.error(APICode.error("存在错误的课件"));
            }
            lessonWareRelationService.update(lessonId, wareIdList);
        }

        return APIResult.ok(courseWareList);
    }
}
