package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.CourseSubTypeEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.dto.CourseDTO;
import cn.rankin.data.api.product.dto.CourseSubRelationDTO;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.data.api.product.vo.CourseSubItemVo;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.*;
import cn.rankin.productservice.utils.DTOConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class CourseService {

    @Autowired
    private CourseRepository courseRepository;

    @Autowired
    private CourseSubRelationService courseSubRelationService;

    @Autowired
    private CourseSupportRelationService courseSupportRelationService;

    @Autowired
    private LessonService lessonService;

    @Autowired
    private SupportService supportService;

    public boolean exists(String code) {
        Long count = courseRepository.countByCode(code);
        return count > 0L;
    }

    public APIResult<Course> getCourse(String courseId) {
        Course course = courseRepository.find(courseId);
        if (course == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        // subItem list
        List<CourseSubItemVo> subItemVoList = findSubItemVo(courseId);
        course.setSubItemList(subItemVoList);

        // support list
        List<Support> supportList = findSupportList(courseId);
        course.setSupportList(supportList);
        return APIResult.ok(course);
    }

    public Course findCourseById(String courseId) {
        Course course = courseRepository.findCourseById(courseId);
        if (course == null) {
            return null;
        }

        // subItem list
        List<CourseSubItemVo> subItemVoList = findSubItemVo(courseId);
        course.setSubItemList(subItemVoList);

        // support list
        List<Support> supportList = findSupportList(courseId);
        course.setSupportList(supportList);
        return course;
    }

    public APIResult<Course> getCourseLessons(String courseId) {
        Course course = courseRepository.find(courseId);
        if (course == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }
        List<CourseSubItemVo> subItemVoList = this.findSubItemVo(courseId);
        course.setSubItemList(subItemVoList);
        return APIResult.ok(course);
    }

    // find sub item by courseId
    public List<CourseSubItemVo> findSubItemVo(String courseId) {
        List<CourseSubItemVo> subItemVoList = new ArrayList<>();

        List<CourseSubRelation> courseSubRelationList = courseSubRelationService.findByCourseId(courseId);
        if (courseSubRelationList == null || courseSubRelationList.size() == 0) {
            return subItemVoList;
        }

        List<String> lessonIdList = new ArrayList<>();
        courseSubRelationList.forEach( relation -> {
            if (CourseSubTypeEnum.LESSON.equals(relation.getType())) {
                lessonIdList.add(relation.getSubId());
            }
        });

        APIResult<List<Lesson>> lessonResult = lessonService.findByIds(lessonIdList);
        List<Lesson> lessonList = lessonResult.getData();
        if (lessonList != null && lessonList.size() > 0) {
            lessonList.forEach(lesson -> subItemVoList.add(convert(lesson)));
        }

        return subItemVoList;
    }

    // find support by courseId
    public List<Support> findSupportList(String courseId) {

        List<CourseSupportRelation> supportRelationList = courseSupportRelationService.findByCourseId(courseId);
        if (supportRelationList == null || supportRelationList.size() == 0) {
            return new ArrayList<>();
        }

        List<String> supportIdList = new ArrayList<>();
        supportRelationList.forEach( relation -> supportIdList.add(relation.getSupportId()));
        APIResult<List<Support>> supportResult = supportService.findByIds(supportIdList);

        return supportResult.getData();
    }

    // 更新二级关系
    @Transactional
    public List<CourseSubItemVo> updateSubRelation(String courseId, List<CourseSubRelationDTO> subRelationDTOList) {
        List<String> lessonIdList = new ArrayList<>();

        if (subRelationDTOList == null) {
            return null;
        }else if (subRelationDTOList.size() == 0) {
            courseSubRelationService.deleteByCourseId(courseId);
            return new ArrayList<>();
        }

        subRelationDTOList.forEach( relation -> {
            if (CourseSubTypeEnum.LESSON.equals(relation.getType())) {
                lessonIdList.add(relation.getId());
            }
        });

        // 查课
        APIResult<List<Lesson>> lessonResult = lessonService.findByIds(lessonIdList);
        List<Lesson> lessonList = lessonResult.getData();
        if (lessonList == null || lessonList.size() != lessonIdList.size()) {
            return null;
        }

        courseSubRelationService.update(courseId, subRelationDTOList);

        List<CourseSubItemVo> subItemVoList = new ArrayList<>();
        lessonList.forEach( lesson -> {
            subItemVoList.add(convert(lesson));
        });

        return subItemVoList;
    }

    // 更新周边关系
    @Transactional
    public List<Support> updateSupportRelation(String courseId, List<String> supportIdList) {
        List<Support> supportList = new ArrayList<>();

        if (supportIdList == null) {
            return null;
        }else if (supportIdList.size() == 0) {
            courseSupportRelationService.deleteByCourseId(courseId);
            return supportList;
        }

        APIResult<List<Support>> supportResult = supportService.findByIds(supportIdList);
        supportList = supportResult.getData();
        if (supportList == null || supportList.size() != supportIdList.size()) {
            return null;
        }

        courseSupportRelationService.update(courseId, supportIdList);
        return supportList;
    }

    @Transactional
    public APIResult<Course> create(CourseDTO courseDTO) {
        String code = courseDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        Course course = DTOConverter.convert(courseDTO);
        Course result = courseRepository.save(course);

        String courseId = course.getId();

        List<CourseSubRelationDTO> courseSubRelationDTOList = courseDTO.getSubItemList();
        List<CourseSubItemVo> subItemVoList = updateSubRelation(courseId, courseSubRelationDTOList);
        course.setSubItemList(subItemVoList);

        List<String> supportIdList = courseDTO.getSupportList();
        List<Support> supportList = updateSupportRelation(courseId, supportIdList);
        course.setSupportList(supportList);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Course> update(CourseDTO courseDTO) {

        Course course = DTOConverter.convert(courseDTO);
        Course result = courseRepository.update(course);

        String courseId = course.getId();

        List<CourseSubRelationDTO> courseSubRelationDTOList = courseDTO.getSubItemList();
        List<CourseSubItemVo> subItemVoList = updateSubRelation(courseId, courseSubRelationDTOList);
        course.setSubItemList(subItemVoList);

        List<String> supportIdList = courseDTO.getSupportList();
        List<Support> supportList = updateSupportRelation(courseId, supportIdList);
        course.setSupportList(supportList);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = courseRepository.deleteById(id);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICode.NOT_EXISTS);
    }

    public APIResult<List<Course>> findByIds(List<String> courseIdList) {
        List<Course> courseList = courseRepository.findByIds(courseIdList, BaseStatusEnum.NORMAL);
        if (courseIdList.size() < 2) {
            return APIResult.ok(courseIdList);
        }
        Map<String, Course> courseMap = ListUtil.convert(courseList, "id", Course.class);
        List<Course> sortCourseList = new ArrayList<>();
        courseIdList.forEach(courseId -> {
            Course course = courseMap.get(courseId);
            if (course != null) {
                sortCourseList.add(course);
            }
        });
        return APIResult.ok(sortCourseList);
    }

    public Course findByIdAndStatus(String id) {
        return courseRepository.findByIdAndStatus(id, BaseStatusEnum.NORMAL);
    }
}
