package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.LessonDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.Lesson;
import cn.rankin.productservice.repository.LessonRepository;
import cn.rankin.productservice.utils.DTOConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class LessonService {

    @Autowired
    private LessonRepository lessonRepository;

    public APIResult<List<Lesson>> findByIds(List<String> lessonIds) {
        List<Lesson> lessonList = lessonRepository.findByIds(lessonIds, BaseStatusEnum.NORMAL);

        if (lessonList == null || lessonList.size() == 0) {
            return APIResult.ok(lessonList);
        }

        Map<String, Lesson> lessonMap = ListUtil.convert(lessonList, "id", Lesson.class);
        List<Lesson> sortLessonList = new ArrayList<>();
        lessonIds.forEach( id -> {
            Lesson lesson = lessonMap.get(id);
            if (lesson != null) {
                sortLessonList.add(lesson);
            }
        });

        return APIResult.ok(sortLessonList);
    }

    public APIResult<Page<Lesson>> search(Lesson lesson, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = lessonRepository.count(lesson);
        Page<Lesson> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Lesson> lessonList = lessonRepository.find(lesson, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(lessonList);

        return APIResult.ok(page);
    }

    public APIResult<Lesson> findById(String id) {
        Lesson lesson = lessonRepository.find(id);
        if (lesson == null) {
            return APIResult.error(APICode.NOT_EXISTS);
        }

        return APIResult.ok(lesson);
    }

    public boolean exists(String code) {
        Long count = lessonRepository.countByCode(code);
        return count > 0L;
    }

    @Transactional
    public APIResult<Lesson> create(LessonDTO lessonDTO) {
        String code = lessonDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        Lesson lesson = convert(lessonDTO);
        Lesson result = lessonRepository.save(lesson);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Lesson> update(LessonDTO lessonDTO) {
        Lesson lesson = DTOConverter.convert(lessonDTO);
        Lesson result = lessonRepository.update(lesson);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        lessonRepository.deleteById(id, BaseStatusEnum.DEL);
        return APIResult.ok(true);
    }

}
