package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.ProductTypeEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.data.api.product.vo.PackageProductVo;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.GoodsRepository;
import cn.rankin.productservice.repository.MerchantProductRepository;
import cn.rankin.productservice.repository.ProductRepository;
import cn.rankin.productservice.repository.TagRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Service
public class MerchantProductService {

    @Autowired
    private MerchantProductRepository merchantProductRepository;

    @Autowired
    private GoodsRepository goodsRepository;

    @Autowired
    private TagRepository tagRepository;

    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private MerchantProductTagRelationService merchantProductTagRelationService;

    @Autowired
    private PackageService packageService;

    @Autowired
    private PackageChangeService packageChangeService;

    @Autowired
    private ProductService productService;


    public APIResult<Page<MerchantProduct>> search(MerchantProduct merchantProduct, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = merchantProductRepository.count(merchantProduct);
        Page<MerchantProduct> page = new Page<>(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<MerchantProduct> merchantProductList = merchantProductRepository.find(merchantProduct, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        if (merchantProductList.size() == 0) {
            page.setList(new ArrayList<>());
            return APIResult.ok(page);
        }

        // map同一产品同一渠道下多个标价商品
        Map<String, List<Goods>> goodsMap = new HashMap<>();

        List<String> productIdList = new ArrayList<>();
        List<String> merchantIdList = new ArrayList<>();
        merchantProductList.forEach(product -> {
            String productId = product.getPid();
            String merchantId = product.getMerchantId();
            productIdList.add(productId);
            merchantIdList.add(merchantId);
        });
        if (productIdList.size() > 0 && merchantIdList.size() > 0) {
            List<Goods> goodsList = goodsRepository.findByPidsAndMerchantIds(productIdList, merchantIdList);

            goodsList.forEach(good -> {
                String productId = good.getPid();
                String merchantId = good.getMerchantId();
                String key = String.format("%s_%s", productId, merchantId);
                if (goodsMap.containsKey(key)) {
                    goodsMap.get(key).add(good);
                } else {
                    List<Goods> tmpList = new ArrayList<>();
                    tmpList.add(good);
                    goodsMap.put(key, tmpList);
                }
            });
        }

        // map同一产品同一渠道下的标签
        Map<String, Tag> tagMap = new HashMap<>();

        List<MerchantProductTagRelation> merchantProductTagRelationList = merchantProductTagRelationService.findByPidsAndMerchantIds(productIdList, merchantIdList);
        List<String> tagIdList = new ArrayList<>();
        Map<String, List<String>> tagIdMap = new HashMap<>();
        merchantProductTagRelationList.forEach(merchantProductTagRelation -> {
            String productId = merchantProductTagRelation.getPid();
            String merchantId = merchantProductTagRelation.getMerchantId();
            String tagId = merchantProductTagRelation.getTagId();
            String key = String.format("%s_%s", productId, merchantId);
            tagIdList.add(tagId);
            if (tagIdMap.containsKey(key)) {
                List<String> tagIds = tagIdMap.get(key);
                tagIds.add(tagId);
            }else {
                List<String> tagIds = new ArrayList<>();
                tagIds.add(tagId);
                tagIdMap.put(key, tagIds);
            }
        });

        if (tagIdList.size() > 0) {
            List<Tag> tagList = tagRepository.findByIds(tagIdList, BaseStatusEnum.NORMAL);
            tagList.forEach(tag -> tagMap.put(tag.getId(), tag));

            // 按sort排序
            for (String key : goodsMap.keySet()) {
                List<Goods> goods = goodsMap.get(key);
                Collections.sort(goods, new Comparator<Goods>() {
                    @Override
                    public int compare(Goods o1, Goods o2) {
                        return o1.getSort().compareTo(o2.getSort());
                    }
                });
            }
        }

        // 挂载标签及标价商品数组
        for (MerchantProduct product : merchantProductList) {
            String productId = product.getPid();

            Map<String, Object> productMap = productService.getProductDict(productId);
            product.setName(String.valueOf(productMap.get("name")));

            String merchantId = product.getMerchantId();
            String key = String.format("%s_%s", productId, merchantId);
            List<Goods> goods = goodsMap.get(key);
            if (!CollectionUtils.isEmpty(goods)) {
                product.setGoods(goods);
            }
            List<String> tagIds = tagIdMap.get(key);
            if (!StringUtils.isEmpty(tagIds)) {
                List<Tag> tags = new ArrayList<>();
                tagIds.forEach(tagId -> {
                    Tag tag = tagMap.get(tagId);
                    if (tag != null) {
                        tags.add(tag);
                    }
                });
                product.setTags(tags);
            }
            ProductTypeEnum type = product.getType();
            if (type.equals(ProductTypeEnum.PACKAGE)) {
                product = this.getProduct(productId, merchantId);
            }
        }

        page.setList(merchantProductList);

        return APIResult.ok(page);
    }

    public MerchantProduct getProduct(String productId, String merchantId) {
        MerchantProduct merchantProduct = merchantProductRepository.findByPidAndMerchantId(productId, merchantId);

        if (merchantProduct == null) {
            return null;
        }

        List<Goods> goodsList = goodsRepository.findByPidAndMerchantId(productId, merchantId);
        merchantProduct.setGoods(goodsList);

        List<MerchantProductTagRelation> merchantProductTagRelationList = merchantProductTagRelationService.findByPidAndMerchantId(productId, merchantId);
        List<String> tagIdList = new ArrayList<>();
        merchantProductTagRelationList.forEach( merchantProductTagRelation -> tagIdList.add(merchantProductTagRelation.getTagId()));
        if (tagIdList.size() > 0) {
            List<Tag> tagList = tagRepository.findByIds(tagIdList, BaseStatusEnum.NORMAL);
            merchantProduct.setTags(tagList);
        }

        ProductTypeEnum type = merchantProduct.getType();
        if (type.equals(ProductTypeEnum.PACKAGE) && goodsList.size() > 0) {
            List<PackageProductVo> productVos = packageService.getPackageProducts(productId);
            merchantProduct.setProducts(productVos);

            List<Goods> packageGoodsList = goodsRepository.findByPkgIdAndMerchantId(productId, merchantId);
            List<Goods> supportGoodsList = new ArrayList<>();
            for (Goods goods : packageGoodsList) {
                if (goods.getType().equals(ProductTypeEnum.SUPPORT)) {
                    supportGoodsList.add(goods);
                }
            }
            merchantProduct.setSpecials(supportGoodsList);
        }

        return merchantProduct;
    }

    @Transactional
    public APIResult change(String productId, String merchantId, BaseStatusEnum status) {
        MerchantProduct merchantProduct = merchantProductRepository.findByPidAndMerchantId(productId, merchantId);
        Product product = productRepository.findByPid(productId);

        if (merchantProduct == null) {
            if (product == null) {
                return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
            }
            merchantProduct = new MerchantProduct();
            merchantProduct.setPid(productId);
            merchantProduct.setCode(product.getCode());
            merchantProduct.setName(product.getName());
            merchantProduct.setType(product.getType());
            merchantProduct.setMerchantId(merchantId);
        }
        merchantProduct.setStatus(status);
        merchantProductRepository.save(merchantProduct);

        if (merchantProduct != null && product.getType().equals(ProductTypeEnum.PACKAGE)) {
            packageChangeService.addPackageInsideGoods(productId, merchantId);
        }

        return APIResult.ok();
    }

    @Transactional
    public APIResult<MerchantProduct> mountTags(String productId, String merchantId, List<String> tagIdList) {
        MerchantProduct merchantProduct = merchantProductRepository.findByPidAndMerchantId(productId, merchantId);
        if (merchantProduct == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        List<Tag> tagList = tagRepository.findByIds(tagIdList);
        if (tagList.size() != tagIdList.size()) {
            return APIResult.error(ProductServiceAPICode.error("存在错误的tag"));
        }else {
            merchantProductTagRelationService.updateByPid(productId, merchantId, tagIdList);
            return APIResult.ok();
        }
    }

    public APIResult<List<MerchantProduct>> findByPidList_bak(List<String> productIdList, String merchantId) {

        if (CollectionUtils.isEmpty(productIdList)) {
            return APIResult.ok(new ArrayList<>());
        }

        List<MerchantProduct> merchantProductList = merchantProductRepository.findByPidsAndMerchantId(productIdList, merchantId, BaseStatusEnum.NORMAL);
        if (merchantProductList.size() == 0) {
            return APIResult.ok(merchantProductList);
        }

        // map同一产品同一渠道下多个标价商品
        Map<String, List<Goods>> goodsMap = new HashMap<>();
        List<Goods> goodsList = goodsRepository.findByPidsAndMerchantId(productIdList, merchantId);

        for (Goods goods : goodsList) {
            String productId = goods.getPid();
            if (goodsMap.containsKey(productId)) {
                List<Goods> tmp = goodsMap.get(productId);
                tmp.add(goods);
            } else {
                List<Goods> tmpList = new ArrayList<>();
                tmpList.add(goods);
                goodsMap.put(productId, tmpList);
            }
        }

        for (MerchantProduct merchantProduct : merchantProductList) {
            String productId = merchantProduct.getPid();
            List<Goods> tmp = goodsMap.get(productId);
            merchantProduct.setGoods(tmp);
        }

        return APIResult.ok(merchantProductList);
    }


    public List<MerchantProduct> findByPidList(List<String> productIdList, String merchantId) {

        if (CollectionUtils.isEmpty(productIdList)) {
            return new ArrayList<>();
        }

        List<MerchantProduct> merchantProductList = merchantProductRepository.findByPidsAndMerchantId(productIdList, merchantId, BaseStatusEnum.NORMAL);
        if (merchantProductList.size() == 0) {
            return new ArrayList<>();
        }

        // map同一产品同一渠道下多个标价商品
        Map<String, List<Goods>> goodsMap = new HashMap<>();
        List<Goods> goodsList = goodsRepository.findByPidsAndMerchantId(productIdList, merchantId);

        for (Goods goods : goodsList) {
            String productId = goods.getPid();
            if (goodsMap.containsKey(productId)) {
                List<Goods> tmp = goodsMap.get(productId);
                tmp.add(goods);
            } else {
                List<Goods> tmpList = new ArrayList<>();
                tmpList.add(goods);
                goodsMap.put(productId, tmpList);
            }
        }

        for (MerchantProduct merchantProduct : merchantProductList) {
            String productId = merchantProduct.getPid();
            List<Goods> tmp = goodsMap.get(productId);

            Map<String, Object> product = productService.getProductDict(productId);
            merchantProduct.setName(String.valueOf(product.get("name")));

            merchantProduct.setGoods(tmp);
        }

        return merchantProductList;
    }


    public List<MerchantProduct> findByPidsAndMerchantId(List<String> productIdList, String merchantId){
        List<MerchantProduct> merchantProductList = merchantProductRepository.findByPidsAndMerchantId(productIdList, merchantId);

        Map<String, MerchantProduct> merchantProductMap = ListUtil.convert(merchantProductList, "pid", MerchantProduct.class);

        List<MerchantProduct> sortMerchantProductList = new ArrayList<>();
        for (String productId : productIdList) {
            MerchantProduct merchantProduct = merchantProductMap.get(productId);
            if (merchantProduct != null) {
                sortMerchantProductList.add(merchantProduct);
            }
        }
        return sortMerchantProductList;
    }

    public MerchantProduct findByPid(String pid) {
        List<MerchantProduct> merchantProductList = merchantProductRepository.findByPid(pid);
        if(null != merchantProductList && merchantProductList.size() > 0 ){
            return merchantProductList.get(0);
        }else{
            return null;
        }
    }
}
