package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.dto.PackageDTO;
import cn.rankin.data.api.product.dto.PackageProductDTO;
import cn.rankin.data.api.product.entity.Package;
import cn.rankin.data.api.product.entity.PackageProductRelation;
import cn.rankin.data.api.product.entity.Product;
import cn.rankin.data.api.product.vo.PackageProductVo;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.PackageRepository;
import cn.rankin.productservice.repository.ProductRepository;
import cn.rankin.productservice.utils.DTOConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class PackageService {

    @Autowired
    private PackageRepository packageRepository;

    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private PackageProductRelationService packageProductRelationService;

    @Autowired
    private PackageChangeService packageChangeService;

    public boolean exists(String code) {
        Long count = packageRepository.countByCode(code);
        return count > 0;
    }

    /**
     *
     * @param pid
     * @return package's children list
     */
    public List<Product> getChildrens(String pid){

        List<PackageProductVo> packageProductVoList = getPackageProducts(pid);

        //get Product by pids
        List<String> pids = new ArrayList<>();

        for(PackageProductVo tmp : packageProductVoList){
            if (null == tmp.getPid() || pid.equals(tmp.getPid())){
                continue;
            }
            pids.add(tmp.getPid());
        }

        if(pids.isEmpty()){
            log.info("Cannot Find Package Childrens, pid={}", pid);
            return new ArrayList<>();
        }

        List<Product> products = productRepository.findByPids(pids);

        if (null == products){
            products = new ArrayList<>();
        }

        return products;
    }

    /**
     *
     * @param pid
     * @return product, with valid package fields
     */
    public Product get(String pid){

        Product pkg = productRepository.findByPid(pid);

        if (null == pkg){
            log.error("Cannot Find Product By pid, pid={}", pid);
            return null;
        }

        List<Product> childrens = getChildrens(pid);

        pkg.setChildrens(childrens);

        return pkg;
    }

    public APIResult<Package> getPackage(String pkgId) {
        Package pkg = packageRepository.find(pkgId);
        if (pkg == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        List<PackageProductVo> packageProductVoList = getPackageProducts(pkgId);
        pkg.setProducts(packageProductVoList);

        return APIResult.ok(pkg);
    }

    public Package findPackageById(String pkgId) {
        Package pkg = packageRepository.findPackageById(pkgId);
        if (pkg == null) {
            return null;
        }

        List<PackageProductVo> packageProductVoList = getPackageProducts(pkgId);
        pkg.setProducts(packageProductVoList);

        return pkg;
    }

    public List<PackageProductVo> getPackageProducts(String pkgId) {
        List<PackageProductVo> packageProductVoList = new ArrayList<>();

        List<PackageProductRelation> relationList = packageProductRelationService.findByPkgId(pkgId);
        if (relationList.size() == 0) {
            return packageProductVoList;
        }

        Map<String, PackageProductRelation> relationMap = new HashMap<>();
        List<String> productIdList = new ArrayList<>();
        relationList.forEach(relation -> {
            String productId = relation.getPid();
            productIdList.add(productId);
            relationMap.put(productId, relation);
        });

        List<Product> productList = productRepository.findByPids(productIdList);
        if (CollectionUtils.isEmpty(productList)) {
            return packageProductVoList;
        }

        Map<String, Product> productMap = ListUtil.convert(productList, "pid", Product.class);
        for (String productId : productIdList) {
            Product product = productMap.get(productId);
            if (product == null) {
                continue;
            }
            PackageProductVo packageProductVo = new PackageProductVo();
            BeanUtils.copyProperties(product, packageProductVo);
            PackageProductRelation relation = relationMap.get(productId);
            packageProductVo.setCpPrice(relation.getCpPrice());
            packageProductVo.setMerchantPrice(relation.getMerchantPrice());
            packageProductVoList.add(packageProductVo);
        }

        return packageProductVoList;
    }

    @Transactional
    public APIResult<Package> create(PackageDTO packageDTO) {
        String code = packageDTO.getCode();
        if (exists(code)) {
            return APIResult.error(ProductServiceAPICode.ALREADY_EXISTS);
        }

        Package pkg = DTOConverter.convert(packageDTO);
        Package result = packageRepository.save(pkg);

        String pkgId = result.getId();
        List<PackageProductDTO> packageProductDTOList = packageDTO.getProducts();

        Boolean updateResult = updateRelation(pkgId, packageProductDTOList);
        if (!updateResult) {
            return APIResult.error(ProductServiceAPICode.error("产品关系更新失败"));
        }

        List<PackageProductVo> packageProductVoList = getPackageProducts(pkgId);
        pkg.setProducts(packageProductVoList);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Package> update(PackageDTO packageDTO) {
        Package pkg = DTOConverter.convert(packageDTO);
        Package result = packageRepository.update(pkg);

        String pkgId = result.getId();
        List<PackageProductDTO> packageProductDTOList = packageDTO.getProducts();

        Boolean updateResult = updateRelation(pkgId, packageProductDTOList);
        if (!updateResult) {
            return APIResult.error(ProductServiceAPICode.error("产品关系更新失败"));
        }

        // 更新课程包内的商品信息
        packageChangeService.updatePackageInsideGoods(pkgId, packageProductDTOList);

        List<PackageProductVo> packageProductVoList = getPackageProducts(pkgId);
        pkg.setProducts(packageProductVoList);
        return APIResult.ok(pkg);
    }

    @Transactional
    public Boolean deletePackage(String pkgId) {
        Integer count = packageRepository.deleteById(pkgId);
        return count > 0;
    }

    @Transactional
    public Boolean updateRelation(String pkgId, List<PackageProductDTO> packageProductDTOList) {
        if (packageProductDTOList == null) {
            return true;
        }else if (packageProductDTOList.size() == 0) {
            packageProductRelationService.deleteByPkgId(pkgId);
            return true;
        }

        List<String> productIdList = new ArrayList<>();
        packageProductDTOList.forEach(packageProductDTO -> productIdList.add(packageProductDTO.getPid()));

        List<Product> productList = productRepository.findByPids(productIdList);
        if (productIdList.size() != productList.size()) {
            return false;
        }

        packageProductRelationService.updateRelation(pkgId, packageProductDTOList);
        return true;
    }

}
