package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.product.dto.SupportDTO;
import cn.rankin.data.api.product.entity.Support;
import cn.rankin.data.api.product.entity.SupportReference;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.SupportRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class SupportService {

    @Autowired
    private SupportRepository supportRepository;

    @Autowired
    private SupportReferenceService supportReferenceService;

    public boolean exists(String code) {
        Long count = supportRepository.countByCode(code);
        return count > 0;
    }

    @Transactional
    public APIResult<Support> create(SupportDTO supportDTO) {
        String code = supportDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        Support support = convert(supportDTO);
        Support result = supportRepository.save(support);

        String supportId = result.getId();
        List<String> supportIdList = supportDTO.getSupportList();
        if (supportIdList != null) {
            List<Support> supportList = updateSelfRelation(supportId, supportIdList);
            result.setSupportList(supportList);
        }

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Support> update(SupportDTO supportDTO) {

        Support support = convert(supportDTO);
        Support result = supportRepository.update(support);

        String supportId = result.getId();
        List<String> supportIdList = supportDTO.getSupportList();
        if (supportIdList != null) {
            List<Support> supportList = updateSelfRelation(supportId, supportIdList);
            result.setSupportList(supportList);
        }
        
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = supportRepository.deleteById(id);
        if (count > 0) {
            return APIResult.ok(true);
        }

        return APIResult.error(APICode.NOT_EXISTS);
    }

    public APIResult<Support> getSupport(String id) {
        Support support = supportRepository.find(id);
        if (support == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        List<Support> supportList = getRelationSupport(support.getId());
        support.setSupportList(supportList);

        return APIResult.ok(support);
    }

    public Support findSupportById(String id) {
        Support support = supportRepository.findSupportById(id);
        if (support == null) {
            return null;
        }

        List<Support> supportList = getRelationSupport(support.getId());
        support.setSupportList(supportList);

        return support;
    }

    @Transactional
    public List<Support> updateSelfRelation(String fid, List<String> tids) {
        if (tids == null) {
            return null;
        }else if (tids.size() == 0) {
            supportReferenceService.deleteBySupportId(fid);
            return new ArrayList<>();
        }

        List<Support> supportList = supportRepository.findByIds(tids);
        if (supportList.size() != tids.size()) {
            return null;
        }

        supportReferenceService.update(fid, tids);

        return supportList;
    }

    public List<Support> getRelationSupport(String fid) {
        List<SupportReference> relationList = supportReferenceService.findByFid(fid);
        if (relationList.size() == 0) {
            return new ArrayList<>();
        }

        List<String> supportIdList = new ArrayList<>();
        relationList.forEach( (relation) -> {
            if (fid.equals(relation.getTid())){
                return;
            }
            supportIdList.add(relation.getTid());
        });

        if (supportIdList.isEmpty()){
            return new ArrayList<>();
        }

        List<Support> supportList = supportRepository.findByIds(supportIdList, BaseStatusEnum.NORMAL);

        return supportList;
    }

    public APIResult<List<Support>> findByIds(List<String> supportIdList) {
        List<Support> supportList = supportRepository.findByIds(supportIdList, BaseStatusEnum.NORMAL);
        if (supportList == null || supportList.size() == 0) {
            return APIResult.ok(supportList);
        }

        Map<String, Support> supportMap = ListUtil.convert(supportList, "id", Support.class);
        List<Support> sortSupportList = new ArrayList<>();
        supportIdList.forEach( id -> {
            Support support = supportMap.get(id);
            if (support != null) {
                sortSupportList.add(support);
            }
        });

        return APIResult.ok(supportList);
    }
}
