package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.product.dto.GroupDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.product.entity.Tag;
import cn.rankin.data.api.product.entity.TagGroup;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.repository.TagGroupRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.LinkedHashMap;
import java.util.List;

import static cn.rankin.productservice.utils.DTOConverter.convert;

@Service
public class TagGroupService {

    @Autowired
    private TagGroupRepository tagGroupRepository;

    @Autowired
    private TagService tagService;

    public boolean exists(String code) {
        Long count = tagGroupRepository.countByCode(code);
        return count > 0;
    }

    public APIResult<Page<TagGroup>> search(TagGroup tagGroup, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = tagGroupRepository.count(tagGroup);

        Page<TagGroup> page = new Page<>(count, pageNo, pageSize);
        if (count == 0) {
            return APIResult.ok(page);
        }

        List<TagGroup> tagGroupList = tagGroupRepository.find(tagGroup, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(tagGroupList);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<TagGroup> create(GroupDTO groupDTO) {
        String code = groupDTO.getCode();
        if (exists(code)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        TagGroup tagGroup = convert(groupDTO);
        TagGroup result = tagGroupRepository.save(tagGroup);
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<TagGroup> update(GroupDTO groupDTO) {
        String groupId = groupDTO.getId();
        TagGroup tagGroup = convert(groupDTO);
        // 更新标签组
        TagGroup result = tagGroupRepository.update(tagGroup);
        // 给组内的标签排序
        List<String> tagIdList = groupDTO.getTagList();
        if (!CollectionUtils.isEmpty(tagIdList)) {
            tagService.sortTag(groupId, tagIdList);
        }
        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String id) {
        Integer count = tagGroupRepository.deleteById(id);
        if (count > 0) {
            return APIResult.ok();
        }
        return APIResult.error(ProductServiceAPICode.error("删除失败"));
    }

    public APIResult<TagGroup> getTagGroup(String groupId) {
        TagGroup tagGroup = tagGroupRepository.find(groupId);
        if (tagGroup == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }

        List<Tag> tagList = tagService.findByGroupId(groupId);
        tagGroup.setTagList(tagList);
        return APIResult.ok(tagGroup);
    }

    public APIResult<TagGroup> findTagGroupByCode(String code, String merchantId) {
        TagGroup tagGroup = tagGroupRepository.findFirstByCode(code, merchantId);
        if (tagGroup == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }
        String groupId = tagGroup.getId();
        return this.getTagGroup(groupId);
    }
}
