package cn.rankin.task.service;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.OrderStatusEnum;
import cn.rankin.data.api.trade.entity.Order;
import cn.rankin.data.api.trade.entity.OrderDetail;
import cn.rankin.data.api.trade.entity.OrderGoods;
import cn.rankin.data.api.trade.entity.OrderProductSnapshot;
import cn.rankin.task.repository.OrderDetailRepository;
import cn.rankin.task.repository.OrderGoodsRepository;
import cn.rankin.task.repository.OrderProductSnapshotRepository;
import cn.rankin.task.repository.OrderRepository;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;

@Slf4j
@Service
public class OrderService {

    @Autowired
    private OrderRepository orderRepository;

    @Autowired
    private OrderDetailRepository orderDetailRepository;

    @Autowired
    private OrderGoodsRepository orderGoodsRepository;

    @Autowired
    private OrderProductSnapshotRepository orderProductSnapshotRepository;

    @Autowired
    private SerialNumberService serialNumberService;

    public List<Order> findOrderForCompleteList(Integer limit) {
        List<Order> orderList = orderRepository.findCompleteOrders(limit);
        if (orderList.size() == 0) {
            return orderList;
        }
        setOrderGoods(orderList);
        return orderList;
    }

    public List<String> findCompleteOrderFromOrderDetail(List<String> orderIdList) {
        List<OrderDetail> orderDetailList = orderDetailRepository.findByOrderIds(orderIdList);
        Set<String> notCompleteOrderIdSet = new HashSet<>();
        for (OrderDetail orderDetail : orderDetailList) {
            String orderId = orderDetail.getOrderId();
            OrderStatusEnum orderStatus = orderDetail.getOrderStatus();
            BaseStatusEnum status = orderDetail.getStatus();
            if (!status.equals(BaseStatusEnum.NORMAL)) {
                continue;
            }
            if (!orderStatus.equals(OrderStatusEnum.COMPLETE)) {
                notCompleteOrderIdSet.add(orderId);
            }
        }

        List<String> completeOrderIdList = new ArrayList<>();
        for (String orderId : orderIdList) {
            if (notCompleteOrderIdSet.contains(orderId)) {
                continue;
            }
            completeOrderIdList.add(orderId);
        }

        return completeOrderIdList;
    }

    public List<OrderDetail> findOrderDetailForCompleteList(Integer limit) {
        List<OrderDetail> orderDetailList = orderDetailRepository.findForCompleteList(limit);
        if (CollectionUtils.isEmpty(orderDetailList)) {
            return orderDetailList;
        }

        List<String> detailIdList = new ArrayList<>();
        orderDetailList.forEach(orderDetail -> detailIdList.add(orderDetail.getId()));

        List<OrderGoods> orderGoodsList = orderGoodsRepository.findByDetailIds(detailIdList);
        Map<String, List<OrderGoods>> goodsMap = new HashMap<>();
        for (OrderGoods orderGoods : orderGoodsList) {
            String detailId = orderGoods.getDetailId();
            List<OrderGoods> tmp = goodsMap.get(detailId);
            if (CollectionUtils.isEmpty(tmp)) {
                tmp = new ArrayList<>();
                tmp.add(orderGoods);
                goodsMap.put(detailId, tmp);
            }else {
                tmp.add(orderGoods);
            }
        }

        for (OrderDetail orderDetail : orderDetailList) {
            String detailId = orderDetail.getId();
            orderDetail.setGoods(goodsMap.get(detailId));
        }

        return orderDetailList;
    }

    public List<Order> findOrderSplitList(Integer limit) {
        List<Order> orderList = orderRepository.findSplitOrders(limit);
        if (CollectionUtils.isEmpty(orderList)) {
            return orderList;
        }
        setOrderGoods(orderList);
        return orderList;
    }

    public void setOrderGoods(List<Order> orderList) {
        List<String> orderIdList = new ArrayList<>();
        for (Order order : orderList) {
            orderIdList.add(order.getId());
        }

        List<OrderGoods> orderGoodsList = orderGoodsRepository.findByOrderIds(orderIdList);
        Map<String, List<OrderGoods>> orderGoodsMap = new HashMap<>();
        for (OrderGoods orderGoods : orderGoodsList) {
            String orderId = orderGoods.getOrderId();
            if (!orderGoodsMap.containsKey(orderId)) {
                List<OrderGoods> tmpList = new ArrayList<>();
                tmpList.add(orderGoods);
                orderGoodsMap.put(orderId, tmpList);
            }else {
                List<OrderGoods> tmpList = orderGoodsMap.get(orderId);
                tmpList.add(orderGoods);
            }
        }

        for (Order order : orderList) {
            String orderId = order.getId();
            List<OrderGoods> tmpList = orderGoodsMap.get(orderId);
            order.setGoods(tmpList);
        }
    }

    @Transactional
    public Boolean split(String orderId, List<OrderDetail> orderDetailList) {
        Integer count = 0;
        if (orderDetailList.size() >= 2) {
            count = orderRepository.split(orderId);
        }else {
            count = orderRepository.process(orderId);
        }

        if (count == 0) {
            log.error("更新原订单分拆状态失败, orderId={}", orderId);
            return Boolean.FALSE;
        }

        for (OrderDetail orderDetail : orderDetailList) {
            orderDetail.setOrderStatus(OrderStatusEnum.FORSEND);
            orderDetail.setSerialNo(serialNumberService.getSlaveOrderSN());
            OrderDetail one = orderDetailRepository.save(orderDetail);
            String detailId = one.getId();
            List<String> goodsIdList = new ArrayList<>();
            List<OrderGoods> orderGoodsList = orderDetail.getGoods();
            for (OrderGoods orderGoods : orderGoodsList) {
                goodsIdList.add(orderGoods.getGoodsId());
            }
            Integer rowCount = orderGoodsRepository.updateDetailIdByIds(detailId, orderId, goodsIdList);
            if (rowCount > 0) {
                log.info("orderId={}, 已成功分拆一单, detailId={}", orderId, detailId);
            }
        }
        return Boolean.TRUE;
    }

    @Transactional
    public Boolean completeDetail(String detailId) {
        Integer count = orderDetailRepository.complete(detailId);
        return count > 0;
    }

    @Transactional
    public Boolean completeOrder(List<String> orderIdList) {
        Integer count = orderRepository.complete(orderIdList);
        if (count == orderIdList.size()) {
            log.error("更新总订单完成状态全部成功, orderIdList={}", JSON.toJSONString(orderIdList));
            return Boolean.TRUE;
        }

        log.error("部分订单更新完成状态失败, orderIdList={}", JSON.toJSONString(orderIdList));
        return Boolean.FALSE;
    }

    @Transactional
    public void insertSnapshots(List<OrderProductSnapshot> snapshotList) {
        orderProductSnapshotRepository.save(snapshotList);
    }
}

