package cn.rankin.tradeservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.OrderSplitStatusEnum;
import cn.rankin.common.utils.enums.OrderStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.trade.dto.OrderSearchDTO;
import cn.rankin.data.api.trade.dto.OrderDTO;
import cn.rankin.data.api.trade.dto.OrderGoodsDTO;
import cn.rankin.data.api.trade.entity.OrderDetail;
import cn.rankin.data.api.trade.entity.OrderGoods;
import cn.rankin.data.api.trade.entity.Order;
import cn.rankin.tradeservice.code.TradeServiceAPICode;
import cn.rankin.tradeservice.repository.OrderDetailRepository;
import cn.rankin.tradeservice.repository.OrderRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.*;

import static cn.rankin.tradeservice.utils.DTOConverter.convert;

@Service
public class OrderService {

    @Autowired
    private OrderRepository orderRepository;

    @Autowired
    private OrderGoodsService orderGoodsService;

    @Autowired
    private OrderDetailRepository orderDetailRepository;

    @Transactional
    public APIResult<Boolean> pay(String orderId) {
        Integer count = orderRepository.pay(orderId);
        if (count > 0) {
            return APIResult.ok();
        }
        return APIResult.error(TradeServiceAPICode.PAY_FAILED);
    }

    @Transactional
    public APIResult<Boolean> cancel(String orderId) {
        Integer count = orderRepository.cancel(orderId);
        if (count > 0) {
            return APIResult.ok();
        }
        return APIResult.error(TradeServiceAPICode.CANCEL_ERROR);
    }

    @Transactional
    public APIResult<Order> create(OrderDTO orderDTO) {
        Order order = convert(orderDTO);
        order.setOrderStatus(OrderStatusEnum.UNPAID);
        order.setStatus(BaseStatusEnum.NORMAL);
        order.setSerialNo(orderDTO.getSerialNo());
        order.setSplitStatus(OrderSplitStatusEnum.NO);
        order = orderRepository.save(order);
        String orderId = order.getId();
        List<OrderGoodsDTO> orderGoodsDTOList = orderDTO.getGoods();
        orderGoodsService.createRelation(orderId, orderGoodsDTOList);
        List<OrderGoods> orderGoodsList = orderGoodsService.findByOrderId(orderId);
        order.setGoods(orderGoodsList);
        return APIResult.ok(order);
    }

    public APIResult<Order> getOrder(String orderId) {
        Order order = orderRepository.find(orderId);
        if (order == null) {
            return APIResult.error(TradeServiceAPICode.NOT_EXISTS);
        }

        List<OrderGoods> orderGoodsList = orderGoodsService.findByOrderId(orderId);
        order.setGoods(orderGoodsList);
        return APIResult.ok(order);
    }

    public APIResult<Page<Order>> search(OrderSearchDTO searchDTO) {
        Specification<Order> specification = new Specification<Order>() {
            @Override
            public Predicate toPredicate(Root<Order> root, CriteriaQuery<?> query, CriteriaBuilder cb) {
                return OrderService.this.toPredicate(root, query, cb, searchDTO);
            }
        };

        Long count = orderRepository.count(specification);
        Page<Order> page = new Page<>(count, searchDTO.getPageNo(), searchDTO.getPageSize());

        Long start = page.getStart();
        if (count == 0) {
            return APIResult.ok(page);
        }

        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);
        List<Order> orderList = orderRepository.find(specification, start, page.getPageSize(), JpaSortUtil.sort(sort));

        List<String> orderIdList = new ArrayList<>();
        orderList.forEach(order -> orderIdList.add(order.getId()));

        List<OrderDetail> orderDetailList = orderDetailRepository.findByOrderIds(orderIdList);
        Map<String, List<OrderDetail>> orderDetailMap = new HashMap<>();
        for (OrderDetail orderDetail : orderDetailList) {
            String orderId = orderDetail.getOrderId();
            List<OrderDetail> tmp = orderDetailMap.get(orderId);
            if (CollectionUtils.isEmpty(tmp)) {
                tmp = new ArrayList<>();
                tmp.add(orderDetail);
                orderDetailMap.put(orderId, tmp);
            }else {
                tmp.add(orderDetail);
            }
        }

        for (Order order : orderList) {
            String orderId = order.getId();
            List<OrderDetail> detailList = orderDetailMap.getOrDefault(orderId, new ArrayList<>());
            order.setDetailList(detailList);
        }

        page.setList(orderList);

        return APIResult.ok(page);
    }

    public Predicate toPredicate(Root<Order> root, CriteriaQuery<?> query, CriteriaBuilder cb, OrderSearchDTO searchDTO) {
        // 存放查询条件
        List<Predicate> list = new ArrayList<Predicate>();
        Predicate tmp = null;

        List<String> userList = searchDTO.getUid();
        if (!CollectionUtils.isEmpty(userList)) {
            tmp = root.get("uid").in(userList);
            list.add(tmp);
        }

        String merchantId = searchDTO.getMerchantId();
        if (!StringUtils.isEmpty(merchantId)) {
            tmp = cb.equal(root.get("merchantId"), merchantId);
            list.add(tmp);
        }
        OrderStatusEnum status = searchDTO.getStatus();
        if (status != null) {
            tmp = cb.equal(root.get("orderStatus"), status);
            list.add(tmp);
        }

        Date beginTime = searchDTO.getBeginTime();
        if (beginTime != null) {
            tmp = cb.greaterThanOrEqualTo(root.get("gmtCreated"), beginTime);
            list.add(tmp);
        }

        Date endTime = searchDTO.getEndTime();
        if (endTime != null) {
            tmp = cb.lessThanOrEqualTo(root.get("gmtCreated"), endTime);
            list.add(tmp);
        }
        // where条件
        return list.isEmpty() ? null : cb.and(list.toArray(new Predicate[list.size()]));
    }
}
